<?

/**
 * Iterator Class
 * Implements iterator for php arrays
 * 
 * @version 1.0
 * @author <a href=mailto:chubrilo@yahoo.com>Dejan Bosanac</a>
 */
class Iterator {

	var $_array = array();
	var $_previous = false;
	var $_next = false;
	var $_dir = true; //forward by default

	/**
 	 * Constructor
	 * Calls initialisation function. @see _initialize()
	 * @param array $array array to be traversed
	 * @access public
 	 */
	function Iterator($array) {
		$this->_initialize($array);
	}
	
	/**
	 * Initialisation function
	 * Sets local value of array and resets it
	 * Called from constructor of this class (@see Iterator()) and 
	 * constructors of all subclasses
	 * @param array $array array to be traversed
	 * @access private
	 */
	function _initialize($array) {
		$this->_array = $array;
		$this->reset();
	} 


	/**
 	 * sets internal pointer to the first element
	 * @access public
 	 */	
	function reset() {
		$this->_next = reset($this->_array);
		$this->_previous = false;
		$this->_dir = true;
	}
	
	/**
 	 * Sets internal pointer to the last element
	 * @access public
 	 */		
	function end() {
		$this->_previous = end($this->_array);
		$this->_next = false;
		$this->_dir = false;
	}
	
	/**
 	 * Sets internal pointer to desired position.
	 * @param integer $position zero-based value for desired position in iterator
	 * @access public
 	 */		
	function seek($position) {
		if ($position >= count($this->_array)) {
			$this->_next = false;
			$this->_previous = end($this->_array);
			return;
		}
		if ($position < 0) {
			$this->_previous = false;
			$this->_next = reset($this->_array);
			return;
		}
		$pos = 0;
		$this->reset();
		
		for($i = 0; $i < $position; $i++) {
			$this->_previous = $this->_next;
			$this->_next = next($this->_array);
		}


		return true;
	}	
	
	/**
	 * Returns next element in the iteration
	 * Template method
	 * @access public
	 */
	function next() {
		$id = $this->_getNextId();
		if ($id)
			return $this->_fetchObject($id);
		else
			return $id;
	}
	
	/**
	 * Returns previous element in the iteration
	 * Template method 
	 * @access public
	 */
	function previous() {
		$id = $this->_getPreviousId();
		if ($id)
			return $this->_fetchObject($id);
		else
			return $id;
	}

	/**
 	 * Returns next element from the array
	 * @access private
 	 */	
	function _getNextId() {
		if (!$this->_dir) {
			next($this->_array);
			$this->_dir = true;
		}
		$this->_previous = $this->_next;
		$result = $this->_previous;
		if ($this->_previous)
			$this->_next = next($this->_array);
		else
			$this->end();
		return $result;
	}
	
	/**
 	 * Returns previous element from the array
	 * @access private
 	 */	
	function _getPreviousId() {
		if ($this->_dir) {
			prev($this->_array);
			$this->_dir = false;
		}
		$this->_next = $this->_previous;
		$result = $this->_next;
		if ($this->_next)
			$this->_previous = prev($this->_array);
		else
			$this->reset();
		return $result;
	}
	
	/**
	 * returns the current element. Should be overidden by subclasses
	 * @param Object $id element of array
	 * @access private
	 */
	function _fetchObject($id) {
		return $id;
	}
	
}


/**
 * Implements iterator for traversing collection of objects
 * for the given array od object identifiers
 * 
 * @version 1.0
 * @author <a href=mailto:chubrilo@yahoo.com>Dejan Bosanac</a>
 */
class ObjectIterator extends Iterator {

	var $_objectName;
	
	/**
	 * Constructor
	 * Calls initialization method (@see Iterator::_initialize()) 
	 * and do some specific configuration
	 * @param array $array array of object ids to be traversed
	 * @param string $objectName class of objects to be created
	 * @access public
	 */
	function ObjectIterator($array, $objectName) {
		$this->_initialize($array);
		$this->_objectName = $objectName;
	}
	
	/**
	 * Returns object with given id
	 * @param Object $id identifier of the object
	 * @return Object next object in the collection
	 * @access private
	 */ 
	function _fetchObject($id) {
		return new $this->_objectName($id);
	}
}

?>